#!/usr/bin/python
import os
CHARACTER_DIRECTORY = os.path.join("game_data","characters")
CHARACTER_FILE_EXTENSION = "txt"
SCENARIO_DIRECTORY = os.path.join("game_data","scenarios")
SCENARIO_FILE_EXTENSION = "txt"
STORY_SEQUENCE_FILE_EXTENSION = "txt"
STORY_SEQUENCE_DIRECTORY = os.path.join("game_data","story")

def LoadCharacterData():
    """Create a dictionary of available character "roles."

    The roles are loaded from a particular directory, one text file per
    role. (Edit with a simple text editor.)
    Lines that are blank, or that start with #, are ignored.
    """
    cast = {}
    character_filenames = [filename for filename in os.listdir(CHARACTER_DIRECTORY) if filename[-3:] == CHARACTER_FILE_EXTENSION]
    for fn in character_filenames:
        f = open(os.path.join(CHARACTER_DIRECTORY,fn))
        text = f.read()
        f.close()
        data = {"name":"????","HP":10,"speed":3,"actions":{}}
        lines = [line for line in text.split("\n") if line and line[0] != "#"]
        for line in lines:
            key, value = line.split("=")
            key = key.lstrip().rstrip()
            value = value.lstrip().rstrip()
            if value.isdigit():
                value = int(value)
            if key and key[0] == "-":
                ## This is an action, in the form of a dictionary.
                key = key[1:] ## Remove "-" marker
                action = {"name":key,"text":"Do something cool.","power":1} ## Defaults
                action_data = value.replace(", ",",")
                action_data = action_data.split(",")
                for pair in action_data:
                    space = pair.find(" ")
                    action_key = pair[:space]
                    action_value = pair[space+1:]
                    if action_value.isdigit():
                        action_value = int(action_value)
                    if action_key == "text":
                        action_value = action_value[1:-1]
                    action[action_key] = action_value
                data["actions"][key] = action ## Add this to the list of actions.
            else:
                data[key] = value
        cast[fn[:-4].title()] = data
    return cast

def LoadScenario(filename):
    """Load an arrangement of terrain, characters, and battle rules.

    The format is a text file divided into three sections."""
    if not filename.endswith(SCENARIO_FILE_EXTENSION):
        filename += "."+SCENARIO_FILE_EXTENSION
    f = open(os.path.join(SCENARIO_DIRECTORY,filename))
    text = f.read()
    f.close()

    try:
        terrain, other = text.split("* CHARACTERS *")
        characters, rules = other.split("* RULES *")
    except:
        raise "Couldn't load scenario file "+filename+"; format is wrong."

    return terrain, characters, rules

def LoadStorySequence(name):
    filename = name + "." + STORY_SEQUENCE_FILE_EXTENSION
    f = open(os.path.join(STORY_SEQUENCE_DIRECTORY,filename))
    text = f.read()
    f.close()
    lines = [line for line in text.split("\n") if line and line[0] != "#"]
    scene = []
    for line in lines:
        if line[0] == "\"":
            ## No speaker name given.
            scene.append({"text":line[1:-1]})
        elif line.startswith("music"):
            song_name = line.split(" ")[1]
            scene.append({"song":song_name})
        else:
            space = line.find(" ")
            speaker = line[:space]
            text = line[space+1:][1:-1]
            scene.append({"text":text,"speaker":speaker})
    return scene

if __name__ == "__main__":
    ## Demo
    cast = LoadCharacterData()
    print "\n\n--- Cast: ---"
    for k in cast:
        print str(cast[k])+"\n"
